@@ -98,6 +98,8 @@
 
 #include "configmake.h"
 
+#include "virdbus.h"
+
 #if HAVE_SASL
 virNetSASLContextPtr saslCtxt = NULL;
 #endif
@@ -584,16 +586,6 @@ error:
 }
 
 
-static int daemonShutdownCheck(virNetServerPtr srv ATTRIBUTE_UNUSED,
-                               void *opaque ATTRIBUTE_UNUSED)
-{
-    if (virStateActive())
-        return 0;
-
-    return 1;
-}
-
-
 /*
  * Set up the logging environment
  * By default if daemonized all errors go to the logfile libvirtd.log,
@@ -713,23 +705,23 @@ error:
 static void
 daemonVersion(const char *argv0)
 {
-    printf ("%s (%s) %s\n", argv0, PACKAGE_NAME, PACKAGE_VERSION);
+    printf("%s (%s) %s\n", argv0, PACKAGE_NAME, PACKAGE_VERSION);
 }
 
 #ifdef __sun
 static int
 daemonSetupPrivs(void)
 {
-    chown ("/var/run/libvirt", SYSTEM_UID, SYSTEM_UID);
+    chown("/var/run/libvirt", SYSTEM_UID, SYSTEM_UID);
 
-    if (__init_daemon_priv (PU_RESETGROUPS | PU_CLEARLIMITSET,
-        SYSTEM_UID, SYSTEM_UID, PRIV_XVM_CONTROL, NULL)) {
+    if (__init_daemon_priv(PU_RESETGROUPS | PU_CLEARLIMITSET,
+                           SYSTEM_UID, SYSTEM_UID, PRIV_XVM_CONTROL, NULL)) {
         VIR_ERROR(_("additional privileges are required"));
         return -1;
     }
 
-    if (priv_set (PRIV_OFF, PRIV_ALLSETS, PRIV_FILE_LINK_ANY, PRIV_PROC_INFO,
-        PRIV_PROC_SESSION, PRIV_PROC_EXEC, PRIV_PROC_FORK, NULL)) {
+    if (priv_set(PRIV_OFF, PRIV_ALLSETS, PRIV_FILE_LINK_ANY, PRIV_PROC_INFO,
+                 PRIV_PROC_SESSION, PRIV_PROC_EXEC, PRIV_PROC_FORK, NULL)) {
         VIR_ERROR(_("failed to set reduced privileges"));
         return -1;
     }
@@ -772,24 +764,128 @@ static int daemonSetupSignals(virNetServerPtr srv)
     return 0;
 }
 
+
+static void daemonInhibitCallback(bool inhibit, void *opaque)
+{
+    virNetServerPtr srv = opaque;
+
+    if (inhibit)
+        virNetServerAddShutdownInhibition(srv);
+    else
+        virNetServerRemoveShutdownInhibition(srv);
+}
+
+
+#ifdef HAVE_DBUS
+static DBusConnection *sessionBus;
+static DBusConnection *systemBus;
+
+static void daemonStopWorker(void *opaque)
+{
+    virNetServerPtr srv = opaque;
+
+    VIR_DEBUG("Begin stop srv=%p", srv);
+
+    ignore_value(virStateStop());
+
+    VIR_DEBUG("Completed stop srv=%p", srv);
+
+    /* Exit libvirtd cleanly */
+    virNetServerQuit(srv);
+}
+
+
+/* We do this in a thread to not block the main loop */
+static void daemonStop(virNetServerPtr srv)
+{
+    virThread thr;
+    virObjectRef(srv);
+    if (virThreadCreate(&thr, false, daemonStopWorker, srv) < 0)
+        virObjectUnref(srv);
+}
+
+
+static DBusHandlerResult
+handleSessionMessageFunc(DBusConnection *connection ATTRIBUTE_UNUSED,
+                         DBusMessage *message,
+                         void *opaque)
+{
+    virNetServerPtr srv = opaque;
+
+    VIR_DEBUG("srv=%p", srv);
+
+    if (dbus_message_is_signal(message,
+                               DBUS_INTERFACE_LOCAL,
+                               "Disconnected"))
+        daemonStop(srv);
+
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
+
+
+static DBusHandlerResult
+handleSystemMessageFunc(DBusConnection *connection ATTRIBUTE_UNUSED,
+                        DBusMessage *message,
+                        void *opaque)
+{
+    virNetServerPtr srv = opaque;
+
+    VIR_DEBUG("srv=%p", srv);
+
+    if (dbus_message_is_signal(message,
+                               "org.freedesktop.login1.Manager",
+                               "PrepareForShutdown"))
+        daemonStop(srv);
+
+    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
+}
+#endif
+
+
 static void daemonRunStateInit(void *opaque)
 {
     virNetServerPtr srv = opaque;
 
+    /* Since driver initialization can take time inhibit daemon shutdown until
+       we're done so clients get a chance to connect */
+    daemonInhibitCallback(true, srv);
+
     /* Start the stateful HV drivers
      * This is deliberately done after telling the parent process
      * we're ready, since it can take a long time and this will
      * seriously delay OS bootup process */
-    if (virStateInitialize(virNetServerIsPrivileged(srv)) < 0) {
+    if (virStateInitialize(virNetServerIsPrivileged(srv),
+                           daemonInhibitCallback,
+                           srv) < 0) {
         VIR_ERROR(_("Driver state initialization failed"));
         /* Ensure the main event loop quits */
         kill(getpid(), SIGTERM);
-        virObjectUnref(srv);
-        return;
+        goto cleanup;
     }
 
+#ifdef HAVE_DBUS
+    /* Tie the non-priviledged libvirtd to the session/shutdown lifecycle */
+    if (!virNetServerIsPrivileged(srv)) {
+
+        sessionBus = virDBusGetSessionBus();
+        if (sessionBus != NULL)
+            dbus_connection_add_filter(sessionBus,
+                                       handleSessionMessageFunc, srv, NULL);
+
+        systemBus = virDBusGetSystemBus();
+        if (systemBus != NULL) {
+            dbus_connection_add_filter(systemBus,
+                                       handleSystemMessageFunc, srv, NULL);
+            dbus_bus_add_match(systemBus,
+                               "type='signal',sender='org.freedesktop.login1', interface='org.freedesktop.login1.Manager'",
+                               NULL);
+        }
+    }
+#endif
     /* Only now accept clients from network */
     virNetServerUpdateServices(srv, true);
+cleanup:
+    daemonInhibitCallback(false, srv);
     virObjectUnref(srv);
 }
 
@@ -884,42 +980,43 @@ static int migrateProfile(void)
 static void
 daemonUsage(const char *argv0, bool privileged)
 {
-    fprintf (stderr,
-             _("\n\
-Usage:\n\
-  %s [options]\n\
-\n\
-Options:\n\
-  -v | --verbose         Verbose messages.\n\
-  -d | --daemon          Run as a daemon & write PID file.\n\
-  -l | --listen          Listen for TCP/IP connections.\n\
-  -t | --timeout <secs>  Exit after timeout period.\n\
-  -f | --config <file>   Configuration file.\n\
-     | --version         Display version information.\n\
-  -p | --pid-file <file> Change name of PID file.\n\
-\n\
-libvirt management daemon:\n"), argv0);
+    fprintf(stderr,
+            _("\n"
+              "Usage:\n"
+              "  %s [options]\n"
+              "\n"
+              "Options:\n"
+              "  -v | --verbose         Verbose messages.\n"
+              "  -d | --daemon          Run as a daemon & write PID file.\n"
+              "  -l | --listen          Listen for TCP/IP connections.\n"
+              "  -t | --timeout <secs>  Exit after timeout period.\n"
+              "  -f | --config <file>   Configuration file.\n"
+              "     | --version         Display version information.\n"
+              "  -p | --pid-file <file> Change name of PID file.\n"
+              "\n"
+              "libvirt management daemon:\n"),
+            argv0);
 
     if (privileged) {
         fprintf(stderr,
-                _("\n\
-  Default paths:\n\
-\n\
-    Configuration file (unless overridden by -f):\n\
-      %s/libvirt/libvirtd.conf\n\
-\n\
-    Sockets:\n\
-      %s/run/libvirt/libvirt-sock\n\
-      %s/run/libvirt/libvirt-sock-ro\n\
-\n\
-    TLS:\n\
-      CA certificate:     %s/pki/CA/caert.pem\n\
-      Server certificate: %s/pki/libvirt/servercert.pem\n\
-      Server private key: %s/pki/libvirt/private/serverkey.pem\n\
-\n\
-    PID file (unless overridden by -p):\n\
-      %s/run/libvirtd.pid\n\
-\n"),
+                _("\n"
+                  "  Default paths:\n"
+                  "\n"
+                  "    Configuration file (unless overridden by -f):\n"
+                  "      %s/libvirt/libvirtd.conf\n"
+                  "\n"
+                  "    Sockets:\n"
+                  "      %s/run/libvirt/libvirt-sock\n"
+                  "      %s/run/libvirt/libvirt-sock-ro\n"
+                  "\n"
+                  "    TLS:\n"
+                  "      CA certificate:     %s/pki/CA/caert.pem\n"
+                  "      Server certificate: %s/pki/libvirt/servercert.pem\n"
+                  "      Server private key: %s/pki/libvirt/private/serverkey.pem\n"
+                  "\n"
+                  "    PID file (unless overridden by -p):\n"
+                  "      %s/run/libvirtd.pid\n"
+                  "\n"),
                 SYSCONFDIR,
                 LOCALSTATEDIR,
                 LOCALSTATEDIR,
@@ -928,24 +1025,24 @@ libvirt management daemon:\n"), argv0);
                 SYSCONFDIR,
                 LOCALSTATEDIR);
     } else {
-        fprintf(stderr,
-                "%s", _("\n\
-  Default paths:\n\
-\n\
-    Configuration file (unless overridden by -f):\n\
-      $XDG_CONFIG_HOME/libvirt/libvirtd.conf\n\
-\n\
-    Sockets:\n\
-      $XDG_RUNTIME_DIR/libvirt/libvirt-sock\n\
-\n\
-    TLS:\n\
-      CA certificate:     $HOME/.pki/libvirt/cacert.pem\n\
-      Server certificate: $HOME/.pki/libvirt/servercert.pem\n\
-      Server private key: $HOME/.pki/libvirt/serverkey.pem\n\
-\n\
-    PID file:\n\
-      $XDG_RUNTIME_DIR/libvirt/libvirtd.pid\n\
-\n"));
+        fprintf(stderr, "%s",
+                _("\n"
+                  "  Default paths:\n"
+                  "\n"
+                  "    Configuration file (unless overridden by -f):\n"
+                  "      $XDG_CONFIG_HOME/libvirt/libvirtd.conf\n"
+                  "\n"
+                  "    Sockets:\n"
+                  "      $XDG_RUNTIME_DIR/libvirt/libvirt-sock\n"
+                  "\n"
+                  "    TLS:\n"
+                  "      CA certificate:     $HOME/.pki/libvirt/cacert.pem\n"
+                  "      Server certificate: $HOME/.pki/libvirt/servercert.pem\n"
+                  "      Server private key: $HOME/.pki/libvirt/serverkey.pem\n"
+                  "\n"
+                  "    PID file:\n"
+                  "      $XDG_RUNTIME_DIR/libvirt/libvirtd.pid\n"
+                  "\n"));
     }
 }
 
@@ -985,8 +1082,8 @@ int main(int argc, char **argv) {
         {0, 0, 0, 0}
     };
 
-    if (setlocale (LC_ALL, "") == NULL ||
-        bindtextdomain (PACKAGE, LOCALEDIR) == NULL ||
+    if (setlocale(LC_ALL, "") == NULL ||
+        bindtextdomain(PACKAGE, LOCALEDIR) == NULL ||
         textdomain(PACKAGE) == NULL ||
         virInitialize() < 0) {
         fprintf(stderr, _("%s: initialization failed\n"), argv[0]);
@@ -1081,7 +1178,7 @@ int main(int argc, char **argv) {
         default:
             VIR_ERROR(_("%s: internal error: unknown flag: %c"),
                       argv[0], c);
-            exit (EXIT_FAILURE);
+            exit(EXIT_FAILURE);
         }
     }
 
@@ -1269,9 +1366,7 @@ int main(int argc, char **argv) {
     if (timeout != -1) {
         VIR_DEBUG("Registering shutdown timeout %d", timeout);
         virNetServerAutoShutdown(srv,
-                                 timeout,
-                                 daemonShutdownCheck,
-                                 NULL);
+                                 timeout);
     }
 
     if ((daemonSetupSignals(srv)) < 0) {
