@@ -26,6 +26,7 @@
 #include "virterror_internal.h"
 #include "util.h"
 #include "memory.h"
+#include "domain_conf.h"
 
 #define VIR_FROM_THIS VIR_FROM_NONE
 
@@ -36,6 +37,7 @@ virNetDevBandwidthParseRate(xmlNodePtr node, virNetDevBandwidthRatePtr rate)
     char *average = NULL;
     char *peak = NULL;
     char *burst = NULL;
+    char *floor = NULL;
 
     if (!node || !rate) {
         virReportError(VIR_ERR_INVALID_ARG, "%s",
@@ -46,40 +48,55 @@ virNetDevBandwidthParseRate(xmlNodePtr node, virNetDevBandwidthRatePtr rate)
     average = virXMLPropString(node, "average");
     peak = virXMLPropString(node, "peak");
     burst = virXMLPropString(node, "burst");
+    floor = virXMLPropString(node, "floor");
 
     if (average) {
         if (virStrToLong_ull(average, NULL, 10, &rate->average) < 0) {
             virReportError(VIR_ERR_CONFIG_UNSUPPORTED,
-                           _("could not convert %s"),
+                           _("could not convert bandwidth average value '%s'"),
                            average);
             goto cleanup;
         }
-    } else {
+    } else if (!floor) {
         virReportError(VIR_ERR_XML_DETAIL, "%s",
-                       _("Missing mandatory average attribute"));
+                       _("Missing mandatory average or floor attributes"));
+        goto cleanup;
+    }
+
+    if ((peak || burst) && !average) {
+        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+                       _("'peak' and 'burst' require 'average' attribute"));
         goto cleanup;
     }
 
     if (peak && virStrToLong_ull(peak, NULL, 10, &rate->peak) < 0) {
         virReportError(VIR_ERR_CONFIG_UNSUPPORTED,
-                       _("could not convert %s"),
+                       _("could not convert bandwidth peak value '%s'"),
                        peak);
         goto cleanup;
     }
 
     if (burst && virStrToLong_ull(burst, NULL, 10, &rate->burst) < 0) {
         virReportError(VIR_ERR_CONFIG_UNSUPPORTED,
-                       _("could not convert %s"),
+                       _("could not convert bandwidth burst value '%s'"),
                        burst);
         goto cleanup;
     }
 
+    if (floor && virStrToLong_ull(floor, NULL, 10, &rate->floor) < 0) {
+        virReportError(VIR_ERR_CONFIG_UNSUPPORTED,
+                       _("could not convert bandwidth floor value '%s'"),
+                       floor);
+        goto cleanup;
+    }
+
     ret = 0;
 
 cleanup:
     VIR_FREE(average);
     VIR_FREE(peak);
     VIR_FREE(burst);
+    VIR_FREE(floor);
 
     return ret;
 }
@@ -87,16 +104,20 @@ cleanup:
 /**
  * virNetDevBandwidthParse:
  * @node: XML node
+ * @net_type: one of virDomainNetType
  *
- * Parse bandwidth XML and return pointer to structure
+ * Parse bandwidth XML and return pointer to structure.
+ * @net_type tell to which type will/is interface connected to.
+ * Pass -1 if this is not called on interface.
  *
  * Returns !NULL on success, NULL on error.
  */
 virNetDevBandwidthPtr
-virNetDevBandwidthParse(xmlNodePtr node)
+virNetDevBandwidthParse(xmlNodePtr node,
+                        int net_type)
 {
     virNetDevBandwidthPtr def = NULL;
-    xmlNodePtr cur = node->children;
+    xmlNodePtr cur;
     xmlNodePtr in = NULL, out = NULL;
 
     if (VIR_ALLOC(def) < 0) {
@@ -110,6 +131,8 @@ virNetDevBandwidthParse(xmlNodePtr node)
         goto error;
     }
 
+    cur = node->children;
+
     while (cur) {
         if (cur->type == XML_ELEMENT_NODE) {
             if (xmlStrEqual(cur->name, BAD_CAST "inbound")) {
@@ -144,6 +167,20 @@ virNetDevBandwidthParse(xmlNodePtr node)
             /* helper reported error for us */
             goto error;
         }
+
+        if (def->in->floor && net_type != VIR_DOMAIN_NET_TYPE_NETWORK) {
+            if (net_type == -1) {
+                /* 'floor' on network isn't supported */
+                virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+                               _("floor attribute isn't supported for "
+                                 "network's bandwidth yet"));
+            } else {
+                virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+                               _("floor attribute is supported only for "
+                                 "interfaces of type network"));
+            }
+            goto error;
+        }
     }
 
     if (out) {
@@ -156,6 +193,13 @@ virNetDevBandwidthParse(xmlNodePtr node)
             /* helper reported error for us */
             goto error;
         }
+
+        if (def->out->floor) {
+            virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+                           _("'floor' attribute allowed "
+                             "only in <inbound> element"));
+            goto error;
+        }
     }
 
     return def;
@@ -175,13 +219,18 @@ virNetDevBandwidthRateFormat(virNetDevBandwidthRatePtr def,
     if (!def)
         return 0;
 
-    if (def->average) {
-        virBufferAsprintf(buf, "  <%s average='%llu'", elem_name,
-                          def->average);
+    if (def->average || def->floor) {
+        virBufferAsprintf(buf, "  <%s", elem_name);
+
+        if (def->average)
+            virBufferAsprintf(buf, " average='%llu'", def->average);
 
         if (def->peak)
             virBufferAsprintf(buf, " peak='%llu'", def->peak);
 
+        if (def->floor)
+            virBufferAsprintf(buf, " floor='%llu'", def->floor);
+
         if (def->burst)
             virBufferAsprintf(buf, " burst='%llu'", def->burst);
         virBufferAddLit(buf, "/>\n");
