@@ -97,6 +97,7 @@ static int virDomainLockManagerAddDisk(virLockManagerPtr lock,
 }
 
 static virLockManagerPtr virDomainLockManagerNew(virLockManagerPluginPtr plugin,
+                                                 const char *uri,
                                                  virDomainObjPtr dom,
                                                  bool withResources)
 {
@@ -118,6 +119,10 @@ static virLockManagerPtr virDomainLockManagerNew(virLockManagerPluginPtr plugin,
           .key = "pid",
           .value = { .i = dom->pid },
         },
+        { .type = VIR_LOCK_MANAGER_PARAM_TYPE_CSTRING,
+          .key = "uri",
+          .value = { .cstr = uri },
+        },
     };
     VIR_DEBUG("plugin=%p dom=%p withResources=%d",
               plugin, dom, withResources);
@@ -152,6 +157,7 @@ error:
 
 
 int virDomainLockProcessStart(virLockManagerPluginPtr plugin,
+                              const char *uri,
                               virDomainObjPtr dom,
                               bool paused,
                               int *fd)
@@ -163,13 +169,14 @@ int virDomainLockProcessStart(virLockManagerPluginPtr plugin,
     VIR_DEBUG("plugin=%p dom=%p paused=%d fd=%p",
               plugin, dom, paused, fd);
 
-    if (!(lock = virDomainLockManagerNew(plugin, dom, true)))
+    if (!(lock = virDomainLockManagerNew(plugin, uri, dom, true)))
         return -1;
 
     if (paused)
         flags |= VIR_LOCK_MANAGER_ACQUIRE_REGISTER_ONLY;
 
-    ret = virLockManagerAcquire(lock, NULL, flags, fd);
+    ret = virLockManagerAcquire(lock, NULL, flags,
+                                dom->def->onLockFailure, fd);
 
     virLockManagerFree(lock);
 
@@ -186,7 +193,7 @@ int virDomainLockProcessPause(virLockManagerPluginPtr plugin,
     VIR_DEBUG("plugin=%p dom=%p state=%p",
               plugin, dom, state);
 
-    if (!(lock = virDomainLockManagerNew(plugin, dom, true)))
+    if (!(lock = virDomainLockManagerNew(plugin, NULL, dom, true)))
         return -1;
 
     ret = virLockManagerRelease(lock, state, 0);
@@ -196,6 +203,7 @@ int virDomainLockProcessPause(virLockManagerPluginPtr plugin,
 }
 
 int virDomainLockProcessResume(virLockManagerPluginPtr plugin,
+                               const char *uri,
                                virDomainObjPtr dom,
                                const char *state)
 {
@@ -205,10 +213,10 @@ int virDomainLockProcessResume(virLockManagerPluginPtr plugin,
     VIR_DEBUG("plugin=%p dom=%p state=%s",
               plugin, dom, NULLSTR(state));
 
-    if (!(lock = virDomainLockManagerNew(plugin, dom, true)))
+    if (!(lock = virDomainLockManagerNew(plugin, uri, dom, true)))
         return -1;
 
-    ret = virLockManagerAcquire(lock, state, 0, NULL);
+    ret = virLockManagerAcquire(lock, state, 0, dom->def->onLockFailure, NULL);
     virLockManagerFree(lock);
 
     return ret;
@@ -224,7 +232,7 @@ int virDomainLockProcessInquire(virLockManagerPluginPtr plugin,
     VIR_DEBUG("plugin=%p dom=%p state=%p",
               plugin, dom, state);
 
-    if (!(lock = virDomainLockManagerNew(plugin, dom, true)))
+    if (!(lock = virDomainLockManagerNew(plugin, NULL, dom, true)))
         return -1;
 
     ret = virLockManagerInquire(lock, state, 0);
@@ -235,6 +243,7 @@ int virDomainLockProcessInquire(virLockManagerPluginPtr plugin,
 
 
 int virDomainLockDiskAttach(virLockManagerPluginPtr plugin,
+                            const char *uri,
                             virDomainObjPtr dom,
                             virDomainDiskDefPtr disk)
 {
@@ -244,13 +253,14 @@ int virDomainLockDiskAttach(virLockManagerPluginPtr plugin,
     VIR_DEBUG("plugin=%p dom=%p disk=%p",
               plugin, dom, disk);
 
-    if (!(lock = virDomainLockManagerNew(plugin, dom, false)))
+    if (!(lock = virDomainLockManagerNew(plugin, uri, dom, false)))
         return -1;
 
     if (virDomainLockManagerAddDisk(lock, disk) < 0)
         goto cleanup;
 
-    if (virLockManagerAcquire(lock, NULL, 0, NULL) < 0)
+    if (virLockManagerAcquire(lock, NULL, 0,
+                              dom->def->onLockFailure, NULL) < 0)
         goto cleanup;
 
     ret = 0;
@@ -271,7 +281,7 @@ int virDomainLockDiskDetach(virLockManagerPluginPtr plugin,
     VIR_DEBUG("plugin=%p dom=%p disk=%p",
               plugin, dom, disk);
 
-    if (!(lock = virDomainLockManagerNew(plugin, dom, false)))
+    if (!(lock = virDomainLockManagerNew(plugin, NULL, dom, false)))
         return -1;
 
     if (virDomainLockManagerAddDisk(lock, disk) < 0)
@@ -290,6 +300,7 @@ cleanup:
 
 
 int virDomainLockLeaseAttach(virLockManagerPluginPtr plugin,
+                             const char *uri,
                              virDomainObjPtr dom,
                              virDomainLeaseDefPtr lease)
 {
@@ -299,13 +310,14 @@ int virDomainLockLeaseAttach(virLockManagerPluginPtr plugin,
     VIR_DEBUG("plugin=%p dom=%p lease=%p",
               plugin, dom, lease);
 
-    if (!(lock = virDomainLockManagerNew(plugin, dom, false)))
+    if (!(lock = virDomainLockManagerNew(plugin, uri, dom, false)))
         return -1;
 
     if (virDomainLockManagerAddLease(lock, lease) < 0)
         goto cleanup;
 
-    if (virLockManagerAcquire(lock, NULL, 0, NULL) < 0)
+    if (virLockManagerAcquire(lock, NULL, 0,
+                              dom->def->onLockFailure, NULL) < 0)
         goto cleanup;
 
     ret = 0;
@@ -326,7 +338,7 @@ int virDomainLockLeaseDetach(virLockManagerPluginPtr plugin,
     VIR_DEBUG("plugin=%p dom=%p lease=%p",
               plugin, dom, lease);
 
-    if (!(lock = virDomainLockManagerNew(plugin, dom, false)))
+    if (!(lock = virDomainLockManagerNew(plugin, NULL, dom, false)))
         return -1;
 
     if (virDomainLockManagerAddLease(lock, lease) < 0)
