@@ -23,6 +23,7 @@
 # define __VIR_PLUGINS_LOCK_DRIVER_H__
 
 # include "internal.h"
+# include "domain_conf.h"
 
 typedef struct _virLockManager virLockManager;
 typedef virLockManager *virLockManagerPtr;
@@ -66,6 +67,7 @@ typedef enum {
 
 enum {
     VIR_LOCK_MANAGER_PARAM_TYPE_STRING,
+    VIR_LOCK_MANAGER_PARAM_TYPE_CSTRING,
     VIR_LOCK_MANAGER_PARAM_TYPE_INT,
     VIR_LOCK_MANAGER_PARAM_TYPE_LONG,
     VIR_LOCK_MANAGER_PARAM_TYPE_UINT,
@@ -84,6 +86,7 @@ struct _virLockManagerParam {
         unsigned long long ul;
         double d;
         char *str;
+        const char *cstr;
         unsigned char uuid[16];
     } value;
 };
@@ -153,6 +156,7 @@ typedef int (*virLockDriverDeinit)(void);
  * - uuid: the domain uuid (uuid)
  * - name: the domain name (string)
  * - pid: process ID to own/owning the lock (unsigned int)
+ * - uri: URI for connecting to the driver the domain belongs to (string)
  *
  * Returns 0 if successful initialized a new context, -1 on error
  */
@@ -215,12 +219,13 @@ typedef int (*virLockDriverAddResource)(virLockManagerPtr man,
  * @manager: the lock manager context
  * @state: the current lock state
  * @flags: optional flags, currently unused
+ * @action: action to take when lock is lost
  * @fd: optional return the leaked FD
  *
  * Start managing resources for the object. This
  * must be called from the PID that represents the
  * object to be managed. If the lock is lost at any
- * time, the PID will be killed off by the lock manager.
+ * time, the specified action will be taken.
  * The optional state contains information about the
  * locks previously held for the object.
  *
@@ -234,6 +239,7 @@ typedef int (*virLockDriverAddResource)(virLockManagerPtr man,
 typedef int (*virLockDriverAcquire)(virLockManagerPtr man,
                                     const char *state,
                                     unsigned int flags,
+                                    virDomainLockFailureAction action,
                                     int *fd);
 
 /**
