@@ -169,7 +169,8 @@ error:
 }
 
 
-static int openvzDefaultConsoleType(const char *ostype ATTRIBUTE_UNUSED)
+static int openvzDefaultConsoleType(const char *ostype ATTRIBUTE_UNUSED,
+                                    const char *arch ATTRIBUTE_UNUSED)
 {
     return VIR_DOMAIN_CHR_CONSOLE_TARGET_TYPE_OPENVZ;
 }
@@ -285,7 +286,7 @@ openvzReadNetworkConf(virDomainDefPtr def,
 
             /*parse string*/
             do {
-                char *next = strchrnul (p, ',');
+                char *next = strchrnul(p, ',');
                 if (STRPREFIX(p, "ifname=")) {
                     /* skip in libvirt */
                 } else if (STRPREFIX(p, "host_ifname=")) {
@@ -456,7 +457,7 @@ openvzReadFSConf(virDomainDefPtr def,
         } else {
             /* Ensure that we can multiply by 1024 without overflowing. */
             if (barrier > ULLONG_MAX / 1024 ||
-                limit > ULLONG_MAX / 1024 ) {
+                limit > ULLONG_MAX / 1024) {
                 virReportSystemError(VIR_ERR_OVERFLOW, "%s",
                                      _("Unable to parse quota"));
                 goto error;
@@ -985,7 +986,7 @@ openvz_readline(int fd, char *ptr, int maxlen)
     char c;
 
     for (n = 1; n < maxlen; n++) {
-        if ( (rc = read(fd, &c, 1)) == 1) {
+        if ((rc = read(fd, &c, 1)) == 1) {
             *ptr++ = c;
             if (c == '\n')
                 break;
