@@ -132,7 +132,7 @@ char *qemuMonitorEscapeArg(const char *in)
     */
 
     for (i = 0; in[i] != '\0'; i++) {
-        switch(in[i]) {
+        switch (in[i]) {
         case '\r':
         case '\n':
         case '"':
@@ -149,7 +149,7 @@ char *qemuMonitorEscapeArg(const char *in)
         return NULL;
 
     for (i = j = 0; in[i] != '\0'; i++) {
-        switch(in[i]) {
+        switch (in[i]) {
         case '\r':
             out[j++] = '\\';
             out[j++] = 'r';
@@ -187,7 +187,7 @@ char *qemuMonitorUnescapeArg(const char *in)
         next = in[i];
         if (in[i] == '\\') {
             ++i;
-            switch(in[i]) {
+            switch (in[i]) {
             case 'r':
                 next = '\r';
                 break;
@@ -283,7 +283,7 @@ qemuMonitorOpenUnix(const char *monitor, pid_t cpid)
             break;
 
         if ((errno == ENOENT || errno == ECONNREFUSED) &&
-            cpid && virProcessKill(cpid, 0) == 0) {
+            (!cpid || virProcessKill(cpid, 0) == 0)) {
             /* ENOENT       : Socket may not have shown up yet
              * ECONNREFUSED : Leftover socket hasn't been removed yet */
             continue;
@@ -297,7 +297,7 @@ qemuMonitorOpenUnix(const char *monitor, pid_t cpid)
 
     if (ret != 0) {
         virReportSystemError(errno, "%s",
-                             _("monitor socket did not show up."));
+                             _("monitor socket did not show up"));
         goto error;
     }
 
@@ -1428,16 +1428,16 @@ qemuMonitorGetBlockInfo(qemuMonitorPtr mon)
 
 struct qemuDomainDiskInfo *
 qemuMonitorBlockInfoLookup(virHashTablePtr blockInfo,
-                           const char *devname)
+                           const char *dev)
 {
     struct qemuDomainDiskInfo *info;
 
-    VIR_DEBUG("blockInfo=%p dev=%s", blockInfo, NULLSTR(devname));
+    VIR_DEBUG("blockInfo=%p dev=%s", blockInfo, NULLSTR(dev));
 
-    if (!(info = virHashLookup(blockInfo, devname))) {
+    if (!(info = virHashLookup(blockInfo, dev))) {
         virReportError(VIR_ERR_INTERNAL_ERROR,
                        _("cannot find info for device '%s'"),
-                       NULLSTR(devname));
+                       NULLSTR(dev));
     }
 
     return info;
@@ -2316,7 +2316,7 @@ int qemuMonitorSendFileHandle(qemuMonitorPtr mon,
     }
 
     if (!mon->hasSendFD) {
-        virReportError(VIR_ERR_CONFIG_UNSUPPORTED,
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED,
                        _("qemu is not using a unix socket monitor, "
                          "cannot send fd %s"), fdname);
         return -1;
@@ -2387,7 +2387,8 @@ int qemuMonitorAddHostNetwork(qemuMonitorPtr mon,
     }
 
     if (mon->json)
-        ret = qemuMonitorJSONAddHostNetwork(mon, netstr);
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
+                       _("JSON monitor should be using AddNetdev"));
     else
         ret = qemuMonitorTextAddHostNetwork(mon, netstr);
 
@@ -2407,7 +2408,7 @@ int qemuMonitorRemoveHostNetwork(qemuMonitorPtr mon,
                                  int vlan,
                                  const char *netname)
 {
-    int ret;
+    int ret = -1;
     VIR_DEBUG("mon=%p netname=%s",
           mon, netname);
 
@@ -2418,7 +2419,8 @@ int qemuMonitorRemoveHostNetwork(qemuMonitorPtr mon,
     }
 
     if (mon->json)
-        ret = qemuMonitorJSONRemoveHostNetwork(mon, vlan, netname);
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
+                       _("JSON monitor should be using RemoveNetdev"));
     else
         ret = qemuMonitorTextRemoveHostNetwork(mon, vlan, netname);
     return ret;
@@ -2539,7 +2541,7 @@ int qemuMonitorAttachDrive(qemuMonitorPtr mon,
           mon, drivestr,
           controllerAddr->domain, controllerAddr->bus,
           controllerAddr->slot, controllerAddr->function);
-    int ret;
+    int ret = 1;
 
     if (!mon) {
         virReportError(VIR_ERR_INVALID_ARG, "%s",
@@ -2548,7 +2550,8 @@ int qemuMonitorAttachDrive(qemuMonitorPtr mon,
     }
 
     if (mon->json)
-        ret = qemuMonitorJSONAttachDrive(mon, drivestr, controllerAddr, driveAddr);
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
+                       _("JSON monitor should be using AddDrive"));
     else
         ret = qemuMonitorTextAttachDrive(mon, drivestr, controllerAddr, driveAddr);
 
@@ -2755,7 +2758,7 @@ qemuMonitorDiskSnapshot(qemuMonitorPtr mon, virJSONValuePtr actions,
                         const char *device, const char *file,
                         const char *format, bool reuse)
 {
-    int ret;
+    int ret = -1;
 
     VIR_DEBUG("mon=%p, actions=%p, device=%s, file=%s, format=%s, reuse=%d",
               mon, actions, device, file, format, reuse);
@@ -2766,17 +2769,12 @@ qemuMonitorDiskSnapshot(qemuMonitorPtr mon, virJSONValuePtr actions,
         return -1;
     }
 
-    if (mon->json) {
+    if (mon->json)
         ret = qemuMonitorJSONDiskSnapshot(mon, actions, device, file, format,
                                           reuse);
-    } else {
-        if (actions || STRNEQ(format, "qcow2") || reuse) {
-            virReportError(VIR_ERR_INVALID_ARG, "%s",
-                           _("text monitor lacks several snapshot features"));
-            return -1;
-        }
-        ret = qemuMonitorTextDiskSnapshot(mon, device, file);
-    }
+    else
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
+                       _("disk snapshot requires JSON monitor"));
     return ret;
 }
 
@@ -2808,7 +2806,7 @@ qemuMonitorDriveMirror(qemuMonitorPtr mon,
         ret = qemuMonitorJSONDriveMirror(mon, device, file, format, speed,
                                          flags);
     else
-        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
                        _("drive-mirror requires JSON monitor"));
     return ret;
 }
@@ -2824,7 +2822,7 @@ qemuMonitorTransaction(qemuMonitorPtr mon, virJSONValuePtr actions)
     if (mon->json)
         ret = qemuMonitorJSONTransaction(mon, actions);
     else
-        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
                        _("transaction requires JSON monitor"));
     return ret;
 }
@@ -2854,7 +2852,7 @@ qemuMonitorBlockCommit(qemuMonitorPtr mon, const char *device,
     if (mon->json)
         ret = qemuMonitorJSONBlockCommit(mon, device, top, base, speed);
     else
-        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
                        _("block-commit requires JSON monitor"));
     return ret;
 }
@@ -2873,7 +2871,7 @@ qemuMonitorDrivePivot(qemuMonitorPtr mon, const char *device,
     if (mon->json)
         ret = qemuMonitorJSONDrivePivot(mon, device, file, format);
     else
-        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
                        _("drive pivot requires JSON monitor"));
     return ret;
 }
@@ -2975,7 +2973,7 @@ int qemuMonitorBlockJob(qemuMonitorPtr mon,
         ret = qemuMonitorJSONBlockJob(mon, device, base, speed, info, mode,
                                       modern);
     else
-        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
                        _("block jobs require JSON monitor"));
     return ret;
 }
@@ -3107,7 +3105,7 @@ int qemuMonitorSystemWakeup(qemuMonitorPtr mon)
     }
 
     if (!mon->json) {
-        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
                        _("JSON monitor is required"));
         return -1;
     }
@@ -3131,7 +3129,7 @@ int qemuMonitorGetVersion(qemuMonitorPtr mon,
     }
 
     if (!mon->json) {
-        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
                        _("JSON monitor is required"));
         return -1;
     }
@@ -3152,7 +3150,7 @@ int qemuMonitorGetMachines(qemuMonitorPtr mon,
     }
 
     if (!mon->json) {
-        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
                        _("JSON monitor is required"));
         return -1;
     }
@@ -3182,7 +3180,7 @@ int qemuMonitorGetCPUDefinitions(qemuMonitorPtr mon,
     }
 
     if (!mon->json) {
-        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
                        _("JSON monitor is required"));
         return -1;
     }
@@ -3204,7 +3202,7 @@ int qemuMonitorGetCommands(qemuMonitorPtr mon,
     }
 
     if (!mon->json) {
-        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
                        _("JSON monitor is required"));
         return -1;
     }
@@ -3226,7 +3224,7 @@ int qemuMonitorGetEvents(qemuMonitorPtr mon,
     }
 
     if (!mon->json) {
-        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
                        _("JSON monitor is required"));
         return -1;
     }
@@ -3249,7 +3247,7 @@ int qemuMonitorGetKVMState(qemuMonitorPtr mon,
     }
 
     if (!mon->json) {
-        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
                        _("JSON monitor is required"));
         return -1;
     }
@@ -3271,7 +3269,7 @@ int qemuMonitorGetObjectTypes(qemuMonitorPtr mon,
     }
 
     if (!mon->json) {
-        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
                        _("JSON monitor is required"));
         return -1;
     }
@@ -3294,7 +3292,7 @@ int qemuMonitorGetObjectProps(qemuMonitorPtr mon,
     }
 
     if (!mon->json) {
-        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
                        _("JSON monitor is required"));
         return -1;
     }
@@ -3315,7 +3313,7 @@ char *qemuMonitorGetTargetArch(qemuMonitorPtr mon)
     }
 
     if (!mon->json) {
-        virReportError(VIR_ERR_CONFIG_UNSUPPORTED, "%s",
+        virReportError(VIR_ERR_OPERATION_UNSUPPORTED, "%s",
                        _("JSON monitor is required"));
         return NULL;
     }
