@@ -44,12 +44,15 @@
 struct uml_driver {
     virMutex lock;
 
-    int privileged;
+    bool privileged;
+    virStateInhibitCallback inhibitCallback;
+    void *inhibitOpaque;
 
     unsigned long umlVersion;
     int nextvmid;
 
     virDomainObjList domains;
+    size_t nactive;
 
     char *configDir;
     char *autostartDir;
