@@ -526,8 +526,10 @@ static int virCgroupSetMemoryUseHierarchy(virCgroupPtr group)
     return rc;
 }
 
-static int virCgroupMakeGroup(virCgroupPtr parent, virCgroupPtr group,
-                              int create, unsigned int flags)
+static int virCgroupMakeGroup(virCgroupPtr parent,
+                              virCgroupPtr group,
+                              bool create,
+                              unsigned int flags)
 {
     int i;
     int rc = 0;
@@ -641,9 +643,9 @@ err:
     return rc;
 }
 
-static int virCgroupAppRoot(int privileged,
+static int virCgroupAppRoot(bool privileged,
                             virCgroupPtr *group,
-                            int create)
+                            bool create)
 {
     virCgroupPtr rootgrp = NULL;
     int rc;
@@ -814,7 +816,7 @@ int virCgroupAddTask(virCgroupPtr group, pid_t pid)
  */
 int virCgroupAddTaskController(virCgroupPtr group, pid_t pid, int controller)
 {
-    if (controller < 0 || controller > VIR_CGROUP_CONTROLLER_LAST)
+    if (controller < 0 || controller >= VIR_CGROUP_CONTROLLER_LAST)
         return -EINVAL;
 
     if (!group->controllers[controller].mountPoint)
@@ -924,8 +926,8 @@ cleanup:
 #if defined HAVE_MNTENT_H && defined HAVE_GETMNTENT_R
 int virCgroupForDriver(const char *name,
                        virCgroupPtr *group,
-                       int privileged,
-                       int create)
+                       bool privileged,
+                       bool create)
 {
     int rc;
     char *path = NULL;
@@ -957,14 +959,32 @@ out:
 #else
 int virCgroupForDriver(const char *name ATTRIBUTE_UNUSED,
                        virCgroupPtr *group ATTRIBUTE_UNUSED,
-                       int privileged ATTRIBUTE_UNUSED,
-                       int create ATTRIBUTE_UNUSED)
+                       bool privileged ATTRIBUTE_UNUSED,
+                       bool create ATTRIBUTE_UNUSED)
 {
     /* Claim no support */
     return -ENXIO;
 }
 #endif
 
+/**
+* virCgroupGetAppRoot:
+*
+* @group: Pointer to returned virCgroupPtr
+*
+* Returns 0 on success
+*/
+#if defined HAVE_MNTENT_H && defined HAVE_GETMNTENT_R
+int virCgroupGetAppRoot(virCgroupPtr *group)
+{
+    return virCgroupNew("/", group);
+}
+#else
+int virCgroupGetAppRoot(virCgroupPtr *group ATTRIBUTE_UNUSED)
+{
+    return -ENXIO;
+}
+#endif
 
 /**
  * virCgroupForDomain:
@@ -979,7 +999,7 @@ int virCgroupForDriver(const char *name ATTRIBUTE_UNUSED,
 int virCgroupForDomain(virCgroupPtr driver,
                        const char *name,
                        virCgroupPtr *group,
-                       int create)
+                       bool create)
 {
     int rc;
     char *path;
@@ -1015,7 +1035,7 @@ int virCgroupForDomain(virCgroupPtr driver,
 int virCgroupForDomain(virCgroupPtr driver ATTRIBUTE_UNUSED,
                        const char *name ATTRIBUTE_UNUSED,
                        virCgroupPtr *group ATTRIBUTE_UNUSED,
-                       int create ATTRIBUTE_UNUSED)
+                       bool create ATTRIBUTE_UNUSED)
 {
     return -ENXIO;
 }
@@ -1034,7 +1054,7 @@ int virCgroupForDomain(virCgroupPtr driver ATTRIBUTE_UNUSED,
 int virCgroupForVcpu(virCgroupPtr driver,
                      int vcpuid,
                      virCgroupPtr *group,
-                     int create)
+                     bool create)
 {
     int rc;
     char *path;
@@ -1060,7 +1080,7 @@ int virCgroupForVcpu(virCgroupPtr driver,
 int virCgroupForVcpu(virCgroupPtr driver ATTRIBUTE_UNUSED,
                      int vcpuid ATTRIBUTE_UNUSED,
                      virCgroupPtr *group ATTRIBUTE_UNUSED,
-                     int create ATTRIBUTE_UNUSED)
+                     bool create ATTRIBUTE_UNUSED)
 {
     return -ENXIO;
 }
@@ -1076,8 +1096,8 @@ int virCgroupForVcpu(virCgroupPtr driver ATTRIBUTE_UNUSED,
  */
 #if defined HAVE_MNTENT_H && defined HAVE_GETMNTENT_R
 int virCgroupForEmulator(virCgroupPtr driver,
-                          virCgroupPtr *group,
-                          int create)
+                         virCgroupPtr *group,
+                         bool create)
 {
     int rc;
     char *path;
@@ -1101,8 +1121,8 @@ int virCgroupForEmulator(virCgroupPtr driver,
 }
 #else
 int virCgroupForEmulator(virCgroupPtr driver ATTRIBUTE_UNUSED,
-                          virCgroupPtr *group ATTRIBUTE_UNUSED,
-                          int create ATTRIBUTE_UNUSED)
+                         virCgroupPtr *group ATTRIBUTE_UNUSED,
+                         bool create ATTRIBUTE_UNUSED)
 {
     return -ENXIO;
 }
@@ -1189,10 +1209,10 @@ int virCgroupSetBlkioDeviceWeight(virCgroupPtr group,
     return ret;
 }
 #else
-        int
- virCgroupSetBlkioDeviceWeight(virCgroupPtr group ATTRIBUTE_UNUSED,
-                                  const char *path ATTRIBUTE_UNUSED,
-                                  unsigned int weight ATTRIBUTE_UNUSED)
+int
+virCgroupSetBlkioDeviceWeight(virCgroupPtr group ATTRIBUTE_UNUSED,
+                              const char *path ATTRIBUTE_UNUSED,
+                              unsigned int weight ATTRIBUTE_UNUSED)
 {
     return -ENOSYS;
 }
@@ -1370,6 +1390,26 @@ int virCgroupGetMemSwapHardLimit(virCgroupPtr group, unsigned long long *kb)
     return ret;
 }
 
+/**
+ * virCgroupGetMemSwapUsage:
+ *
+ * @group: The cgroup to get mem+swap usage for
+ * @kb: The mem+swap amount in kilobytes
+ *
+ * Returns: 0 on success
+ */
+int virCgroupGetMemSwapUsage(virCgroupPtr group, unsigned long long *kb)
+{
+    long long unsigned int usage_in_bytes;
+    int ret;
+    ret = virCgroupGetValueU64(group,
+                               VIR_CGROUP_CONTROLLER_MEMORY,
+                               "memory.memsw.usage_in_bytes", &usage_in_bytes);
+    if (ret == 0)
+        *kb = usage_in_bytes >> 10;
+    return ret;
+}
+
 /**
  * virCgroupSetCpusetMems:
  *
