@@ -44,22 +44,24 @@ VIR_ENUM_DECL(virCgroupController);
 
 int virCgroupForDriver(const char *name,
                        virCgroupPtr *group,
-                       int privileged,
-                       int create);
+                       bool privileged,
+                       bool create);
+
+int virCgroupGetAppRoot(virCgroupPtr *group);
 
 int virCgroupForDomain(virCgroupPtr driver,
                        const char *name,
                        virCgroupPtr *group,
-                       int create);
+                       bool create);
 
 int virCgroupForVcpu(virCgroupPtr driver,
                      int vcpuid,
                      virCgroupPtr *group,
-                     int create);
+                     bool create);
 
 int virCgroupForEmulator(virCgroupPtr driver,
                          virCgroupPtr *group,
-                         int create);
+                         bool create);
 
 int virCgroupPathOfController(virCgroupPtr group,
                               int controller,
@@ -92,6 +94,7 @@ int virCgroupSetMemorySoftLimit(virCgroupPtr group, unsigned long long kb);
 int virCgroupGetMemorySoftLimit(virCgroupPtr group, unsigned long long *kb);
 int virCgroupSetMemSwapHardLimit(virCgroupPtr group, unsigned long long kb);
 int virCgroupGetMemSwapHardLimit(virCgroupPtr group, unsigned long long *kb);
+int virCgroupGetMemSwapUsage(virCgroupPtr group, unsigned long long *kb);
 
 enum {
     VIR_CGROUP_DEVICE_READ  = 1,
