@@ -14,7 +14,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library;  If not, see
+ * License along with this library.  If not, see
  * <http://www.gnu.org/licenses/>.
  *
  */
@@ -35,4 +35,10 @@ int virTypedParameterAssign(virTypedParameterPtr param, const char *name,
                             int type, /* TYPE arg */ ...)
     ATTRIBUTE_RETURN_CHECK;
 
+int virTypedParameterAssignFromStr(virTypedParameterPtr param,
+                                   const char *name,
+                                   int type,
+                                   const char *val)
+    ATTRIBUTE_RETURN_CHECK;
+
 #endif /* __VIR_TYPED_PARAM_H */
