@@ -6,7 +6,8 @@
 #include "testutilsxen.h"
 #include "domain_conf.h"
 
-static int testXenDefaultConsoleType(const char *ostype ATTRIBUTE_UNUSED)
+static int testXenDefaultConsoleType(const char *ostype,
+                                     const char *arch ATTRIBUTE_UNUSED)
 {
     if (STREQ(ostype, "hvm"))
         return VIR_DOMAIN_CHR_CONSOLE_TARGET_TYPE_SERIAL;
@@ -27,7 +28,7 @@ virCapsPtr testXenCapsInit(void) {
         "xenpv"
     };
 
-    uname (&utsname);
+    uname(&utsname);
     if ((caps = virCapabilitiesNew(utsname.machine,
                                    0, 0)) == NULL)
         return NULL;
