@@ -36,7 +36,7 @@ testTypes(const void *data ATTRIBUTE_UNUSED)
     bool res;
 
 #define virAssertCmpInt(a, op, b) \
-    if (!((a) op (b))) \
+    if (!(a op b)) \
         return -1;
     virAtomicIntSet(&u, 5);
     u2 = virAtomicIntGet(&u);
